/*
 * Reference:
 * [1] http://mxr.mozilla.org/seamonkey/source/toolkit/components/places/public/nsINavHistoryService.idl
 * [2] http://mxr.mozilla.org/seamonkey/source/toolkit/components/places/public/nsINavBookmarksService.idl
 * [3] https://developer.mozilla.org/en-US/docs/Places_Developer_Guide
 */

//Interface 
exports.getTree = getTree;
exports.get = get;
exports.remove = remove;
exports.removeTree = removeTree;
exports.move = move;
exports.create = create;
exports.update = update;

exports.onMoved = function(){ return onMovedEvent;};
exports.onRemoved = function(){ return onRemoveEvent;};
exports.onCreated = function() { return onCreateEvent;};
exports.onChanged = function() { return onChangeEvent;};
exports.onChildrenReordered = function () { return onChildrenReorderedEvent;};

exports.removeBookmarkObserver = removeBookmarkObserver;
exports.addBookmarkObserver = addBookmarkObserver;

const {Cc,Ci} = require("chrome");
var bookmarksService = Cc["@mozilla.org/browser/nav-bookmarks-service;1"].getService(Ci.nsINavBookmarksService);
var historyService =  Cc["@mozilla.org/browser/nav-history-service;1"].getService(Ci.nsINavHistoryService);
var URI = Cc["@mozilla.org/network/io-service;1"].getService(Ci.nsIIOService);
var folders = [bookmarksService.bookmarksMenuFolder, bookmarksService.toolbarFolder, bookmarksService.unfiledBookmarksFolder];

var timers = require("sdk/timers");
var print_msg = require("../util").print_msg;
function getTree(callback){
    var result = constructRootNode();
    if(callback != null) {
        //timers.setTimeout(function(){callback([result]);},0);
        callback([result]);
    }
};

function constructRootNode() {
    var rootNode={id:0,children:[]};
    var query = historyService.getNewQuery();

    for (var i = 0; i < folders.length; i++) {    
        var childNode = {};    
        query.setFolders([folders[i]], 1);
        childNode.title = bookmarksService.getItemTitle(folders[i]);
        childNode.index = bookmarksService.getItemIndex(folders[i]);
        childNode.parentId = 0;
        childNode.id = folders[i];
        //result is type of nsINavHistoryResult.
        var result = historyService.executeQuery(query, historyService.getNewQueryOptions());
        //folderNode is type of nsIHistoryContainerResultNode.
        var folderNode = result.root;
        
        folderNode.containerOpen = true;
        childNode.children = constructOnChildren(folderNode, folders[i]);
        folderNode.containerOpen = false;
        rootNode.children.push(childNode);
    }
   
    return rootNode;
}


function constructOnChildren(folderNode, nodeId){
    var childNodes=[];
    for(var i=0; i<folderNode.childCount;i++) {
       var childNode = {};
       
       var child = folderNode.getChild(i);
       
       var title = child.title;
       var id = child.itemId;
       var index = child.bookmarkIndex;

       childNode.id = id;
       childNode.title = title;
       childNode.index = index;
       childNode.parentId = nodeId;
	  
       var type = child.type;
       if(type == Ci.nsINavHistoryResultNode.RESULT_TYPE_URI) {
          var uri = child.uri;
          childNode.url = uri;
          ////console.log(uri);
       }
       else if(type == Ci.nsINavHistoryResultNode.RESULT_TYPE_FOLDER) {
          child.QueryInterface(Ci.nsINavHistoryContainerResultNode);
          
          //open the folder, and iterate over it's content.
          child.containerOpen = true;
          //iterate on children of this node.
          childNode.children = constructOnChildren(child, id);
          child.containerOpen = false;
       }
       childNodes.push(childNode);       
    }

    return childNodes;
}

function create(nodeInfo, callback) {
    //console.log("boomark create:"+JSON.stringify(nodeInfo));
    var bookmarkNode = {};
    
	try {
	    if(nodeInfo.url != null) {
	        var bookmarkURI = URI.newURI(nodeInfo.url, null, null);
	        var bookmarkId = bookmarksService.insertBookmark(
	                       nodeInfo.parentId, // The id of the folder the bookmark will be placed in.
	                       bookmarkURI,             // The URI of the bookmark - an nsIURI object.
	                       nodeInfo.index == null ? bookmarksService.DEFAULT_INDEX : nodeInfo.index, // The position of the bookmark in its parent folder.
	                       nodeInfo.title);    // The title of the bookmark. 
	       bookmarkNode.id = bookmarkId;
	       nodeInfo.url = nodeInfo.url;
	    }    
	    else {
	        var folderId = bookmarksService.createFolder(
	                          nodeInfo.parentId, // The id of the folder the new folder will be placed in.
	                          nodeInfo.title,       // The title of the new folder.
	                          nodeInfo.index == null ? bookmarksService.DEFAULT_INDEX : nodeInfo.index);    // The position of the new folder in its parent folder.
	        bookmarkNode.id = folderId;
	    }
	    
	    bookmarkNode.parentId = nodeInfo.parentId;
	    bookmarkNode.title = nodeInfo.title;
	
	}
	catch(e) {
		//console.log("[Create bookmark error]"+e.message);
	}

    if(callback != null) {
       timers.setTimeout(function(){callback(bookmarkNode);}, 0);
    }
}

function update(id, changes, callback) {
    //console.log("boomark update:"+id+","+JSON.stringify(changes));
    try{
	    if(changes.title != null){
	        bookmarksService.setItemTitle(id, changes.title)
	    }   
	    if(changes.url != null) {
	        var bookmarkURI = URI.newURI(changes.url, null, null);
	       bookmarksService.changeBookmarkURI(id, bookmarkURI);
	    }    
    }
 	catch(e) {
 		//console.log("[Update bookmark error]"+e.message);
 	}
 	if(callback != null) {
 		timers.setTimeout(callback, 0);
 	}
}

function remove(id, callback) {
   //console.log("boomark remove:"+id);
   try{
   		bookmarksService.removeItem(id);   
   }
   catch(e) {
   		//console.log("[Remove bookmark error]" + e.message);
   }
   if(callback != null) {
      timers.setTimeout(callback.bind(this,id), 0);
   }   
}

function removeTree(id,callback){
    remove(id,callback);
}

function move(id, destination, callback) {
    //console.log("bookmark move:"+id+JSON.stringify(destination));
    try{
    	bookmarksService.moveItem(id, destination.parentId, destination.index);    
    }
    catch(e) {
    	//console.log("[Move bookmark error]"+e.message);
    }
    if(callback != null) {
        timers.setTimeout(function(){callback();}, 0);
    }
}

function get(idList, callback){
        
   var ids = {};
   for(var id in idList){
       ids[idList[id]]=0;
   }
   var nodes=[];

    var root = constructRootNode();
    findInit(idList.length);
    findNode(ids, nodes, root);
    if(callback != null) {
        timers.setTimeout(function(){ callback(nodes.length > 0 ? nodes : null);}, 0);
    }
}

var find_num = 0;
var find_total = 0;

function findInit(len){
    find_num = 0;
    find_total = len;
}

function findNode(ids, nodes, root){
    if(ids.hasOwnProperty(root.id)) {
        nodes.push(root);
        find_num += 1;
    }
    if(find_num < find_total) {
        if(root.children != null) {
            for(var idx in root.children){
                findNode(ids, nodes, root.children[idx]);
            }
        }
    }

}
/*
 *	Reference:
 *	[1]  http://people.mozilla.org/~dietrich/places/interfacens_i_nav_bookmark_observer.html
 *	[2]  https://developer.mozilla.org/en-US/docs/XPCOM_Interface_Reference/nsINavBookmarkObserver	
 */
var observer = {
    onItemMoved: function(id, oldParent, oldIndex, newParent, newIndex) {
    	//console.log("[BookmarkMoved]");
    	if(EventHandler.moveHandler != null) {
    		var moveInfo = {
    			index : oldIndex,
    			oldParentId : oldParent,
    			parentId : newParent,
    			oldIndex : newIndex      
    		};   
    		EventHandler.moveHandler(id, moveInfo);
    	}       
    },
    onItemRemoved: function(id, parent, index) {
    	//console.log("[BookmarkRemoved]");
    	if(EventHandler.removeHandler != null) {
    		var removeInfo = {
    			index : index,
    			parentId : parent 
    		};
    		EventHandler.removeHandler(id, removeInfo);
    	}
    },
    onItemChanged: function(id, property, annotationProperty, value) {
    	//console.log("[BookmarkChanged]");
    	if(EventHandler.changeHandler != null) {
    		var changeInfo = {};
    		EventHandler.changeHandler(id, changeInfo);
    	}
    },
    onItemAdded: function(id, parent, index) {
    	//console.log("[BookmarkAdd]");
    	if(EventHandler.changeHandler != null) {
    		var addInfo = {
    			parentId : parent,
    			index : index
    		};
	    	//console.log("[BookmarkAddCallback]");
	    	//console.log(JSON.stringify(addInfo));
    		EventHandler.addHandler(id, addInfo);
    	}
    }
};

function addBookmarkObserver(){
	bookmarksService.addObserver(observer, false);	
}

function removeBookmarkObserver(){
	bookmarksService.removeObserver(observer);
}

var EventHandler ={
    moveHandler : null,
    removeHandler : null,
    changeHandler : null,
    addHandler: null
}

var onMovedEvent = {
    addListener: function(handler) {
        EventHandler.moveHandler = handler;             
    }
};

var onRemoveEvent = {
	addListener: function(handler) {
		EventHandler.removeHandler = handler;
	}
};

var onChangeEvent = {
	addListener: function(handler) {
		EventHandler.changeHandler = handler;
	}	
};

var onCreateEvent = {
	addListener: function(handler) {
		EventHandler.addHandler = handler;
	}	
};

var onChildrenReorderedEvent = {
    addListener: function(handlder) {
    	//console.log("children reorder listener is not implemented.");
    }
};
